/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file tsi.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring touch sense input (TSI). 
*
******************************************************************************/
#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "common.h"
#include "tsi.h"
#include "s08p_lite_board.h"
#include "printf.h"

/******************************************************************************
* Constants and macros
******************************************************************************/

/*****************************************************************************
* Local types
******************************************************************************/
TOUCHKEY_TSI_MAPPING_T TouchKey_TSI[] =
{
    //TSI_channel;baseline_delta;touch_delta;touchkey_state;touch_debounce;release_debounce;touch_threshold;baseline;current_value;    
    {TSI_2, 5, 300},               //KEY 0
    {TSI_15, 5, 300},              //KEY 1

    
    //do NOT move {0xFF, 0xFF, 0xFF, 0xFF} which indicates the end of the array
    {0xFF, 0xFF, 0xFF, 0xFF}                //the end of the array
};

uint8_t u8TouchKeyIdInScan = 0;

unsigned long int STICK_TOUCHKEY_TIMEOUT = 10000;

//max debounce for touch, determine the touch filter time/response time
unsigned char MAX_TOUCHKEY_DEBOUNCE = 3;      

//max debounce for baseline, determine the baseline tracking time
unsigned short MAX_TOUCHKEY_BASELINE_DEBOUNCE = 10;

void TSI_SetMeasuredChannel(uint8_t channel)
{
    TSI_CS3_TSICH   = channel;
}

void TSI_StartSoftwareTrigger(void)
{
    TSI_CS0_SWTS    = 1;
}

uint8_t TSI_GetEndOfScanFlag()
{
    return (TSI_CS0_EOSF);
}

void TSI_ClearEndOfScanFlag(void)
{
    TSI_CS0_EOSF = 1;
}

uint16_t TSI_GetCounter(void)
{
    return (uint16_t)(TSI_CNT);
}

//calibrate TSI idle value, save as TSI baseline
void TSI_Calibrate(void)
{
    unsigned char touchkey_id = 0;
    unsigned char delay_i, delay_j;

    //Initial baseline for each touch key
    while ((TouchKey_TSI[touchkey_id].touchkey_touch_delta!=0xFF) && (TouchKey_TSI[touchkey_id].touchkey_baseline_delta!=0xFF) && (touchkey_id<MAX_TOUCH_KEY_NUM))
    {           
        //initiate TouchKey_TSI structure
        TouchKey_TSI[touchkey_id].touchkey_state                      = 0;
        TouchKey_TSI[touchkey_id].touchkey_touch_debounce      = 0;
        TouchKey_TSI[touchkey_id].touchkey_release_debounce 	= 0;
        TouchKey_TSI[touchkey_id].touchkey_baseline                  = 0;
        TouchKey_TSI[touchkey_id].touchkey_current_value          = 0;

        delay_i = 0;
        delay_j = 0;

        TSI_SetMeasuredChannel(TouchKey_TSI[touchkey_id].TSI_channel); 
        TSI_StartSoftwareTrigger();

        while (!TSI_GetEndOfScanFlag())
        {
            //if can't get end-of-scan set flag, start time-out counter
            delay_j ++;
            if (delay_j >= 1000)
            {
                delay_j = 0;
                delay_i ++;
            }

            if (delay_i >= 20)
            {
                //if TSI scan do NOT ends after about 50ms, before WatchDog reset(2s).
                //Re-initiate TSI module, and force this round of TSI scan to an end
                //TSI_reinit();
                return;
            }
        }
        
		TSI_ClearEndOfScanFlag();

		TouchKey_TSI[touchkey_id].touchkey_baseline = TSI_GetCounter();  

		touchkey_id ++;
    }
}

//Initiate TSI register, like enable TSI module, set charge current/voltage, oscillator frequency
//track TSI baseline through TSI calibration function.
void TSI_Init(void)
{
    unsigned char touchkey_id = 0;
    
    TSI_CS0 = 0;                                //initiate TSI CS0
    TSI_CS0_STM = TSI_SOFTWARE_TRIGGER_SCAN;    //software trigger scan
    
    TSI_CS1 = 0;                                //initiate TSI CS1
    TSI_CS1_NSCN = 31;                          //scan 32 times per TSI channel
    TSI_CS1_PS = 1;                             //electrode oscillator frequency divided by 2

    TSI_CS2 = 0;                                //initiate TSI CS2
    TSI_CS2_EXTCHRG = TSI_CHARGE_CURRENT_32uA;  //electrode charge and discharge current value is 32 uA
    TSI_CS2_REFCHRG = TSI_CHARGE_CURRENT_64uA;  //reference charge and discharge current value is 64 uA

    while ((TouchKey_TSI[touchkey_id].touchkey_touch_delta!=0xFF) && (TouchKey_TSI[touchkey_id].touchkey_baseline_delta!=0xFF) && (touchkey_id<MAX_TOUCH_KEY_NUM))
    {  
        if (TouchKey_TSI[touchkey_id].TSI_channel < 8)
        {
            TSI_PEN0 |= (1<<TouchKey_TSI[touchkey_id].TSI_channel);                            //Enable all TSI channels
        }
        else
        {
            TSI_PEN1 |= (1<<(TouchKey_TSI[touchkey_id].TSI_channel-8));
        }
        
        touchkey_id ++;
    }

    TSI_CS0_TSIEN = 1;                          //Module enabled but disabled in STOP mode, interrupt disabled

    TSI_Calibrate();


}


/*
 * Function: detect self-mode touch key.
 * Return value:
 * TouchKeyEvent,    return key event, which is the same as TSI event.
 * u8TouchKeyIdInScan,       return valid touched key id, ranges from 0 - 99 
 */
unsigned char TSI_Task(unsigned char *current_touchkey_id)
{
    unsigned char TSI_channel;

    unsigned short TSI_sampleResult = 0;
    
    unsigned char touchkey_state_temp;
    unsigned short touchkey_release_debounce_temp;
    unsigned short touchkey_baseline_temp;
    unsigned char touchkey_touch_debounce_temp;

    unsigned short touchkey_touch_threshold_temp;
    unsigned short touchkey_idle_high;
    unsigned short touchkey_idle_low;

    unsigned char delay_i = 0;
    unsigned char delay_j = 0;
    
    TOUCHKEY_EVENT_E TouchKeyEvent = TOUCHKEY_EVENT_IDLE;

    TOUCHKEY_TSI_MAPPING_T *key;
    
    key = &TouchKey_TSI[u8TouchKeyIdInScan];
    TSI_channel = key->TSI_channel;

    if(TSI_channel == 0xFF)
    {
        //reach the end of TouchKey_TSI[] array.
        //re-start the TSI scan from the first key
    	u8TouchKeyIdInScan = 0;
    	
        key = &TouchKey_TSI[u8TouchKeyIdInScan];
        TSI_channel = key->TSI_channel;
    }

    if (TSI_channel > MAX_TSI_NUM)
    {
        //set the default value if the tsi channel is illegal value.
        *current_touchkey_id = 0xFF;
        return 0;
    }
    
    TSI_SetMeasuredChannel(TSI_channel); 
    TSI_StartSoftwareTrigger();

    while (!TSI_GetEndOfScanFlag())
    {
        //if can't get end-of-scan set flag, start time-out counter
        delay_j ++;
        if (delay_j >= 100)
        {
            delay_j = 0;
            delay_i ++;
        }

        if (delay_i >= 100)
        {
            //if TSI scan do NOT ends after about 50ms, before WatchDog reset(2s).
            //Re-initiate TSI module, and force this round of TSI scan to an end
            //TSI_reinit();
            //return 0xFF;
        }
    }

    TSI_ClearEndOfScanFlag();

    TSI_sampleResult = TSI_GetCounter();  
        
    if (TSI_sampleResult == 65535)
    {
        //there's an error occurred during the current tsi channel
        TSI_sampleResult = 0;
    }

    touchkey_state_temp 				= key->touchkey_state;
    touchkey_touch_debounce_temp 	= key->touchkey_touch_debounce;
    touchkey_release_debounce_temp 	= key->touchkey_release_debounce;
    touchkey_baseline_temp 		= key->touchkey_baseline;
    touchkey_touch_threshold_temp 	= touchkey_baseline_temp + key->touchkey_touch_delta;
    touchkey_idle_high		= touchkey_baseline_temp + key->touchkey_baseline_delta;
    touchkey_idle_low		= touchkey_baseline_temp - key->touchkey_baseline_delta;

    if (touchkey_state_temp == TOUCHKEY_STATE_IDLE)
    {
    	TouchKeyEvent = TOUCHKEY_EVENT_IDLE;
    	
        if (TSI_sampleResult > touchkey_touch_threshold_temp)
        {
           //there's a finger touch
            touchkey_touch_debounce_temp ++;
            touchkey_release_debounce_temp = 0;

            if (touchkey_touch_debounce_temp > MAX_TOUCHKEY_DEBOUNCE)
            {    
                touchkey_touch_debounce_temp 	= 0;
                touchkey_release_debounce_temp 	= 0;
                TouchKeyEvent                   = TOUCHKEY_EVENT_TOUCH;       //indicates the TSI channel is touched
                touchkey_state_temp              = TOUCHKEY_STATE_TOUCHED;
            }
        }
        else if (TSI_sampleResult > touchkey_idle_high)
        {
            touchkey_touch_debounce_temp = 0;				
            touchkey_release_debounce_temp ++;

            if (touchkey_release_debounce_temp > MAX_TOUCHKEY_BASELINE_DEBOUNCE)
            {
                touchkey_release_debounce_temp 	= 0;	
                touchkey_baseline_temp = touchkey_baseline_temp + 1;
            }				
        }
        else if (TSI_sampleResult < touchkey_idle_low)
        {
            touchkey_touch_debounce_temp = 0;				
            touchkey_release_debounce_temp ++;

            if (touchkey_release_debounce_temp > MAX_TOUCHKEY_BASELINE_DEBOUNCE)
            {
                touchkey_release_debounce_temp	= 0;
                touchkey_baseline_temp = touchkey_baseline_temp - 1;
            }
        }
        else
        {
            touchkey_touch_debounce_temp 	= 0;
            touchkey_release_debounce_temp 	= 0;
        }
    }
    else if (touchkey_state_temp == TOUCHKEY_STATE_TOUCHED)
    {
        TouchKeyEvent = TOUCHKEY_EVENT_TOUCHED;                          //indicates the TSI channel is touched

        if (TSI_sampleResult > touchkey_touch_threshold_temp)
        {
            //indicates the TSI channel is Stick Touched            
            touchkey_touch_debounce_temp ++;
               
            if (touchkey_touch_debounce_temp > STICK_TOUCHKEY_TIMEOUT)
            {
               touchkey_touch_debounce_temp = 0;
               TouchKeyEvent = TOUCHKEY_EVENT_STICK;   
            }   
        }
        else
        {
            //condition: finger touch release
            touchkey_touch_debounce_temp		= 0;
            touchkey_release_debounce_temp ++;

            if (touchkey_release_debounce_temp > MAX_TOUCHKEY_DEBOUNCE)
            {
                touchkey_release_debounce_temp   = 0;
                TouchKeyEvent                   = TOUCHKEY_EVENT_RELEASE;
                touchkey_state_temp              = TOUCHKEY_STATE_RELEASED;
            }
        }
    }
    else if (touchkey_state_temp == TOUCHKEY_STATE_RELEASED)
    {
        //restore TOUCHKEY_STATE_IDLE automatically
        TouchKeyEvent = TOUCHKEY_EVENT_RELEASED;

        touchkey_release_debounce_temp ++;

        if (touchkey_release_debounce_temp > 5)
        {
            //delay to assure this key is sent
            touchkey_release_debounce_temp   = 0;
            TouchKeyEvent                   = TOUCHKEY_EVENT_IDLE;
            touchkey_state_temp              = TOUCHKEY_STATE_IDLE;
        }
    }
    else
    {
        //key state can NOT go here
        TouchKeyEvent                   = TOUCHKEY_EVENT_IDLE;
        touchkey_state_temp              = TOUCHKEY_STATE_IDLE;
    }

    key->touchkey_state	        		= touchkey_state_temp;
    key->touchkey_current_value 	       	= TSI_sampleResult;
    key->touchkey_baseline              	= touchkey_baseline_temp;
    key->touchkey_touch_threshold	   	= touchkey_touch_threshold_temp;
    key->touchkey_touch_debounce 	    = touchkey_touch_debounce_temp;
    key->touchkey_release_debounce       = touchkey_release_debounce_temp;

    //sample the next key
    *current_touchkey_id = u8TouchKeyIdInScan;
    u8TouchKeyIdInScan ++;
    
    return ((unsigned char)TouchKeyEvent);
}

void TSI_Test(void)
{
    uint8_t u8TouchKeyEvent = 0;
    uint8_t pad1IsTouchedFlag = 0;
    uint8_t pad2IsTouchedFlag = 0;
    uint8_t u8TouchKeyId = 0;
    printf("TSI test: BEGINS\n");

    printf("TSI test: touch pad 1 or 2 on board now\n");
    printf("TSI test: pass test until pad 1, 2 are touched\n");

    while (1)
    {
        u8TouchKeyEvent = TSI_Task(&u8TouchKeyId);

        if (u8TouchKeyEvent == TOUCHKEY_EVENT_TOUCH)
        {
            if (u8TouchKeyId == 0)
            {
                pad1IsTouchedFlag = 1;
                LED6_On();
            }
            else
            {
                pad2IsTouchedFlag = 1;
                LED1_On();
            }
        }
        else if (u8TouchKeyEvent == TOUCHKEY_EVENT_RELEASE)
        {
            if (u8TouchKeyId == 0)
            {
                LED6_Off();
            }
            else
            {
                LED1_Off();
            }
        }
        else
        {
            //
        }

        if (pad1IsTouchedFlag && pad2IsTouchedFlag)
        {
            printf("TSI test: PASSED\n");
            printf("\n");
            return;
        }
    }
}

